"use strict";

const path = require("path");
const semver = require("semver");
const colors = require("colors/safe");
const userHome = require("user-home");
const pathExists = require("path-exists").sync;
const commander = require("commander");
const pkg = require("../package.json");
const log = require("../utils/log");
const exec = require("../models/exec");

const constant = require("./const");
const program = new commander.Command();

async function core() {
  try {
    await prepare();
    registerCommand();
  } catch (e) {
    log.error(e.message);
    if (program.debug) {
      console.log(e);
    }
  }
}

async function prepare() {
  checkPkgVersion();
  checkRoot();
  checkUserHome();
  checkEnv();
  await checkGlobalUpdate();
}

async function checkGlobalUpdate() {
  const currentVersion = pkg.version;
  const npmName = pkg.name;
  const { getNmpSemverVersion } = require("../utils/get-npm-info");
  const newVersion = await getNmpSemverVersion(currentVersion, npmName);
  if (newVersion && semver.gt(newVersion, currentVersion)) {
    log.warn(
      `Update tip:`,
      colors.yellow(
        `current version: ${currentVersion}, new version:${newVersion},
        ${colors.green(`run: npm install ${npmName} -g`)}`
      )
    );
  }
}

// Configure env variables
function checkEnv() {
  const dotenv = require("dotenv");
  const dotenvPath = path.resolve(userHome, ".env");
  if (pathExists(dotenvPath)) {
    dotenv.config({
      path: dotenvPath,
    });
  }
  createDefaultConfig();
}

function createDefaultConfig() {
  const cliConfig = {
    home: userHome,
  };
  if (process.env.CLI_HOME) {
    cliConfig["cliHome"] = path.join(userHome, process.env.CLI_HOME);
  } else {
    cliConfig["cliHome"] = path.join(userHome, constant.DEFAULT_CLI_HOME);
  }
  process.env.CLI_HOME_PATH = cliConfig.cliHome;
}

function checkUserHome() {
  if (!userHome || !pathExists(userHome)) {
    throw new Error(
      colors.red(`The current user home directory does not exist`)
    );
  }
}

function checkRoot() {
  const rootCheck = require("root-check");
  rootCheck();
}

function checkPkgVersion() {
  log.notice("cli", pkg.version);
}

function registerCommand() {
  program
    .name(Object.keys(pkg.bin)[0])
    .usage("<command> [options]")
    .version(pkg.version)
    .option("-d, --debug", "debug mode", false)
    .option("-tp, --targetPath <path>", "local debug file path");

  program
    .command("init [projectName]")
    .description("create a new project")
    .option("-f, --force", "force project creation")
    .action(exec);

  program.on("option:debug", function (value) {
    if (program.debug) {
      process.env.LOG_LEVELE = "verbose";
    } else {
      process.env.LOG_LEVELE = "info";
    }
    log.level = process.env.LOG_LEVELE;
  });

  program.on("option:targetPath", function () {
    if (program.targetPath) {
      process.env.CLI_TARGET_PATH = program.targetPath;
    }
  });

  program.on("command:*", function (command) {
    log.error(`command ${command} does not exist`);
    const availableCommands = program.commands.map((cmd) => cmd.name());
    if (availableCommands.length > 0) {
      console.log(`available commands: ${availableCommands.join(", ")}`);
    }
  });
  program.parse(process.argv);

  if (program.args && program.args.length < 1) {
    program.outputHelp();
    console.log();
  }
}

module.exports = core;
